/*
** Module   : Database.java
** Abstract : Information required to connect to a database
**
** Copyright (c) 2004-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 ECF 20070911 ADD   @35296 Created initial version. Information required
**                               to connect to a database.
** 002 ECF 20071002 CHG   @35329 Exposed all methods as public.
** 003 ECF 20071009 CHG   @35598 Reworked to better support remote databases.
**                               This support is not yet complete.
** 004 ECF 20071119 CHG   @35906 Completed rework to support remote databases.
** 005 CA  20080425 CHG   @38111 Added dirty flag to track the dirty
**                               databases.
** 006 ECF 20080510 CHG   @38481 Minor changes. Intern database name string
**                               and additional toString() debug output.
** 007 OM  20130917 Added userid member and accessors for it.
*/

package com.goldencode.p2j.persist;

import java.net.*;

/**
 * A collection of information required to connect to a database, including
 * the database's physical name and the host address (including port) of the
 * P2J server instance which is the authoritative controller of this database.
 * For the current (i.e., non-remote) server instance, host address will be
 * <code>null</code>.
 * <p>
 * Instances of this class are used extensively in the persistence framework
 * as tokens to access additional information that is database specific.
 * 
 * @author ECF
 */
public final class Database
{
   /** Physical name of database */
   private final String name;
   
   /** Socket address of P2J server which is authoritative for database */
   private InetSocketAddress socketAddress = null;
   
   /** Is this database managed by the local P2J server instance? */
   private Boolean local = null;
   
   /** Flag used to keep the dirty status of this database config */
   private boolean dirty = false;

   /** 
    * The current userid for this database in this database context.
    * It is the empty string for unconnected databases (not the case - I guess); and the 
    * OS user name for databases for which the userid was not explicitly set in the CONNECT 
    * statement or using SETUSERID function.
    */
   private String userid = "";
   
   /**
    * Constructor which accepts the physical database name.
    * 
    * @param   name
    *          Physical name of database.  May not be <code>null</code>.
    */
   public Database(String name)
   {
      this(name, null, -1);
   }
   
   /**
    * Constructor which accepts the physical database name, host name/address,
    * and port number.
    * 
    * @param   name
    *          Physical name of database.  May not be <code>null</code>.
    * @param   host
    *          Host name or address of P2J server which is authoritative for
    *          database.  <code>null</code> indicates host is not relevant.
    * @param   port
    *          Port of P2J server which is authoritative for database.
    *          -1 indicates port is unknown.  May not be less than -1 or
    *          greater than 65535.
    */
   public Database(String name, String host, int port)
   {
      if (name == null || name.trim().length() == 0)
      {
         throw new NullPointerException("Database name may not be null");
      }
      
      if (port < -1 || port > 65535)
      {
         throw new IllegalArgumentException("Invalid port number:  " + port);
      }
      
      if (host != null && port >= 0)
      {
         this.socketAddress = new InetSocketAddress(host, port);
      }
      
      this.name = name.intern();
   }
   
   /**
    * Package-private constructor which accepts the physical database name
    * and an indication as to whether this database is locally managed.
    * <p>
    * This constructor is intended for use by other framework classes which
    * must ensure that the determination of whether this class is locally
    * managed is not deferred until lazily initialized by the {@link
    * #isLocal()} method.
    * 
    * @param   name
    *          Physical name of database.  May not be <code>null</code>.
    * @param   local
    *          <code>true</code> to indicate database is locally managed, else
    *          <code>false</code>.
    */
   Database(String name, boolean local)
   {
      this(name, null, -1);
      
      this.local = (local ? Boolean.TRUE : Boolean.FALSE);
   }
   
   /**
    * Calculate a hash code for this object.
    * 
    * @return  Hash code.
    */
   public int hashCode()
   {
      int result = 17;
      result = 37 * result + name.toUpperCase().hashCode();
      if (socketAddress != null)
      {
         result = 37 * result + socketAddress.hashCode();
      }
      
      result = 37 * result + 
                 (dirty ? Boolean.TRUE.hashCode() : Boolean.FALSE.hashCode());
      
      return result;
   }

   /**
    * Test this object for equality (equivalence) with the given object.
    * 
    * @param   o
    *          Object to test.
    * 
    * @return  <code>true</code> if this object is the same instance as the
    *          given object, or it is considered equivalent to the given
    *          object.  <code>Database</code> instances are equivalent if
    *          they contain the same database name and P2J server address.
    */
   public boolean equals(Object o)
   {
      if (this == o)
      {
         // Same instance.
         return true;
      }
      
      if (!(o instanceof Database))
      {
         // Impossible to be equivalent.
         return false;
      }
      
      Database that = (Database) o;
      
      // This comparison assumes that physical database names have been
      // canonicalized prior to instantiation of these Database instances.
      if (!this.name.toUpperCase().equals(that.name.toUpperCase()))
      {
         return false;
      }
      
      if (this.dirty != that.dirty)
      {
         return false;
      }
      
      // This comparison will fail if different hostnames actually would
      // resolve to the same address.
      InetSocketAddress addr1 = this.socketAddress;
      InetSocketAddress addr2 = that.socketAddress;
      boolean sameAddr = (addr1 == null || addr2 == null
                          ? addr1 == addr2         // both null
                          : addr1.equals(addr2));  // equivalent
      
      return sameAddr;
   }
   
   /**
    * Produce a string representation of the internal state of this object.
    * 
    * @return  String representation of this object.
    */
   public String toString()
   {
      StringBuilder buf = new StringBuilder();
      if (local == null)
      {
         buf.append("?");
      }
      else if (local.booleanValue())
      {
         buf.append("local");
      }
      else
      {
         buf.append(socketAddress);
      }
      buf.append("/");
      buf.append(name);
      buf.append(dirty ? "/dirty" : "/primary");
      
      return buf.toString();
   }
   
   /**
    * Get the physical database name.
    * 
    * @return  Physical database name.
    */
   public String getName()
   {
      return name;
   }
   
   /**
    * Get the host address (including port number) of the P2J server instance
    * which is authoritative for the database instance represented by this
    * object.
    * 
    * @return  P2J server address.
    */
   public InetSocketAddress getSocketAddress()
   {
      return socketAddress;
   }
   
   /**
    * Indicate whether the current P2J server instance is authoritative for
    * the database instance represented by this object.
    * 
    * @return  <code>true</code> if the current P2J server instance is
    *          authoritative;  <code>false</code> if this database is
    *          controlled by a remote P2J server instance.  Note that "remote"
    *          and "local" are logical concepts and have nothing to do with
    *          the physical location of the database or P2J server instance.
    */
   public boolean isLocal()
   {
      if (local == null)
      {
         local = (DatabaseManager.isAuthoritative(this) ? Boolean.TRUE : Boolean.FALSE);
      }
      
      return local.booleanValue();
   }
   
   /**
    * Get the dirty status of this database.
    * 
    * @return dirty status.
    */
   public boolean isDirty()
   {
      return dirty;
   }
   
   /**
    * Set the dirty status of this database.
    * 
    * @param   dirty
    *          <code>true</code> if this is a dirty database
    */
   public void setDirty(boolean dirty)
   {
      this.dirty = dirty;
   }

   /**
    * Returns the current userid for this database in this database context.
    *  
    * @return  current userid
    */
   public String getCurrentUserid()
   {
      return userid;
   }

   /**
    * Sets the currently authentified userid. The only authority to set this is an instance of 
    * <code>MetadataSecurityOps</code> and only after checking the user/password combination
    * against the _User meta-table.
    * 
    * Notes:
    *  * The CONNECT statement with -U and -P parameters will have to call SecurityOps to verify 
    *       if the user/password combination if correct. (At this moment the ConnectionManager
    *       does not support these parameters, yet).
    *  * The command-line parameters should be investigated.
    * 
    * @param   userid
    *          The newly userid for this database context.
    */
   public void setCurrentUserid(String userid)
   {
      this.userid = userid;
   }
}
